# Analysis of subtypes of IT, PT and CT neurons
# Five sections are involved:
# (1) heatmap based on regional projection strength
# (2) classification based on hierarchical clustering analysis
# (3) soma distribution of each subtypes in each region
# (4) 3D soma distribution of each subtype
# (5) 3D morphology of each subtypes
#
# 2020/6/1-GAO Le


library(ComplexHeatmap)
library(nat)

library(gplots)
library(ggplot2)
library(ggridges)
library(ggpubr)


setwd("~/Documents/YanLab/Task_Local/Classification_IT_Ipsilateral_and_contralateral_isocortex_striatum/")

if(!exists("SourceRegion")) 
  source("../shared_R/public.r")

# Read files.
proj = read.table("./PFC/projection_normalized_min_max.txt")
proj_name_neuron = read.table("./PFC/name_neuron.txt")
proj_name_region = read.table("./PFC/name_region.txt")
laminar_neuron = read.table("./PFC/laminar_position.txt")
label_CC_neuron = read.table("./PFC/label_CC.txt")
soma_position = read.table("./PFC/soma.txt")

index_level_3 = read.csv("./index_level_3_10000.csv", header = T, sep = " ")

proj_as_matrix = as.matrix(proj[index_level_3$index, ])
proj_name_neuron = data.frame(V1 = proj_name_neuron$V1[index_level_3$index])
laminar_neuron = laminar_neuron[index_level_3$index, 1]
label_CC_neuron = label_CC_neuron[index_level_3$index, 1]
soma_position = soma_position[index_level_3$index, ]

rownames(proj_as_matrix) <- as.character(proj_name_neuron[,1])
colnames(proj_as_matrix) <- as.character(proj_name_region[,1])


NumNeuron = dim(proj_as_matrix)[1]
NumTargetRegion = dim(proj_as_matrix)[2]

print(NumNeuron)
print(NumTargetRegion)

LabelSourceRegion = func_getLabelRegion(NumNeuron = NumNeuron, 
                                        proj_name_neuron = proj_name_neuron)
LabelSubtype = func_getLabelSubtype(NumNeuron = NumNeuron, 
                                    proj_name_neuron = proj_name_neuron)
NeuronDistribution = func_getRegionDistribution(proj_name_neuron[,1])

SWCPath = func_getSWCPath(proj_name_neuron)


## Heatmap
row_ha = HeatmapAnnotation(region = LabelSourceRegion, 
                           subtype = LabelSubtype,
                           col = list(region = ColorSourceRegion,
                                      subtype = c("ITi"="#FF0000","ITc"="#0000FF")
                                     ),
                           which = "column",
                           show_annotation_name = TRUE,
                           annotation_name_gp = gpar(fontsize = 7),
                           annotation_name_rot = 90,
                           
                           annotation_legend_param = list(title_gp = gpar(fontsize = 7, fontface = "plain"),
                                                          labels_gp = gpar(fontsize = 7))
                           )


# Test Complex Heatmap.
ht = Heatmap(matrix = t(proj_as_matrix[,c(seq(10,16,1), seq(19, 25))]), 
             #ht = Heatmap(matrix = t(proj_as_matrix[,1:25]), 
                          
             name = "Normalized projection strength",
             heatmap_legend_param	= list(title_gp = gpar(fontsize = 5),
                                         labels_gp = gpar(fontsize = 5)),
             
             clustering_distance_rows = "spearman",
             clustering_method_rows = "ward.D2",
             
             clustering_distance_columns = "spearman",
             clustering_method_columns = "ward.D2",

             show_column_names = FALSE,
             
             #column_split = 6,
             
             #top_annotation = row_ha,
             
             cluster_rows = TRUE,
             
             #cluster_columns = columnDend,
             
             #row_split = LabelSourceRegion,
             
             width = unit(12, "cm"), 
             height = unit(5, "cm"),
             
             row_names_side = "left",
             row_names_rot = 0,
             row_names_gp = gpar(fontsize = 7),
             
             column_names_side = "bottom",
             column_names_rot = 90,
             column_names_gp = gpar(fontsize = 7),
             use_raster = TRUE
             )
draw(ht)

#
columnDend = column_dend(ht)
columnOrder = column_order(ht)


##
NumCluster = 6

Label_Cluster = cutree(tree = as.hclust(columnDend), k = NumCluster)

labelOnDend = unique(Label_Cluster[columnOrder])

Label_Cluster_real = Label_Cluster
for(iLabel in 1:NumCluster) {
  Label_Cluster_real[which(Label_Cluster == labelOnDend[iLabel])] = iLabel
}


## Regional distribution of each cluster
par("fin" = c(4,6), "mai" = c(0,0,0,0), "mar" = c(0.5,5,0.5,3))

par(mfrow = c(NumCluster, 1))
for (iCluster in 1:NumCluster) {
  tmpNameList = proj_name_neuron[which(iCluster == Label_Cluster_real),]
  
  # color of clusters in dendrogram
  #tmpColor = leaveColor[which(unname(which(clusters==iCluster)[1]) == hc$order)]
  
  tmpVectorDis = func_getRegionDistribution(tmpNameList)
  tmpVectorRatio = tmpVectorDis / as.numeric(NeuronDistribution)

  barplot2(
      tmpVectorRatio,
      names.arg = SourceRegion,
      col = ColorCluster[iCluster],
      space = 0.1,
      axes = FALSE
      )
  maxLabel = round(max(tmpVectorRatio)/0.01 - 1, 1) * 0.01
  
  print(maxLabel - max(tmpVectorRatio))
  
  axis(side = 2, at = maxLabel *seq(0, 1, by = 1), font = 0.5, las = 1, gap.axis = 0.001)
  title(ylab = iCluster)
  
}

#write.table(tmpLabel_Cluster, "/tmp/soma_IT_ipsilateral_Isocortex_K6.txt", col.names = FALSE)


## regional distribution
contingency_table = func_getClusterContingencyTable(NumCluster, Label_Cluster_real, proj_name_neuron, NeuronDistribution)
distribution_table = apply(contingency_table, 2, function(x) x/sum(x))

rownames(distribution_table) = as.character(paste0("IT_", seq(1, NumCluster, 1)))

matrix_p = func_getEnrichmentPValue(NumClusters = NumCluster, contingency_table)

binary_p = as.matrix(matrix_p > 0 & matrix_p < 0.001)

NumOfNeuronInEachCluster = apply(contingency_table, 1, sum)


library("circlize")
col_fun = colorRamp2(c(0,0.8), c("white", "red"))

Heatmap(distribution_table,
        name = "Ratio",
        #col = col_fun,
        heatmap_legend_param	= list(title_gp = gpar(fontsize = 7),
                                    labels_gp = gpar(fontsize = 7)),
        
        rect_gp = gpar(type = "none"),
        
        cell_fun = function(j,i,x,y,width,height,fill) {
          
          if(binary_p[i,j]) {
            grid.circle(x = x, y = y, r = 0.035,
                        gp = gpar(fill = col_fun(distribution_table[i, j]), col = NA))
            grid.points(x = x, y = y, pch = 8, size = unit(5, "pt"))
          }
          else {
            grid.circle(x = x, y = y, r = 0.035,
                        gp = gpar(fill = col_fun(distribution_table[i, j]), col = NA))
          }
          
        },
        # row_split = seq(1,9,1),
        # column_split = seq(1,11,1),
        
        row_title = NULL,
        column_title = NULL,
        
        cluster_rows = FALSE,
        cluster_columns = FALSE,
        border = FALSE,
        
        
        width = unit(6, "cm"), 
        height = unit(6, "cm"),
        
        row_names_side = "left",
        row_names_rot = 0,
        row_names_gp = gpar(fontsize = 7),
        
        column_names_side = "top",
        column_names_rot = 45,
        column_names_gp = gpar(fontsize = 7),
        use_raster = TRUE
)



## select example neurons
View(proj_as_matrix[which(Label_Cluster_real == 4),])


## Write soma position for each cluster


write.table(x = soma_position, file = "./level_3_soma_position.txt", col.names = F)
write.table(x = Label_Cluster_real, file = "./level_3_cluster_label.txt", col.names = F)



## visualization (population)
index_neuron = base::intersect(which(Label_Cluster_real == 7),
                               which(LabelSourceRegion == "MOs"))
print(proj_name_neuron[index_neuron,1])

test_neuronlist = read.neurons(paths = SWCPath[index_neuron], 
                               neuronnames = proj_name_neuron[index_neuron,1])

test_neuronlist_flipped = func_flipRight2Left(test_neuronlist, proj_name_neuron[index_neuron,1])

open3d()
plot3d(test_neuronlist_flipped, WithNodes = F, WithLine = F, soma = 100)
shade3d(mesh_root, col = "grey", alpha = 0.1)
view3d(userMatrix = userMatrix_IT_DV, fov = 0)


laminar_MOs_6 = laminar_neuron$V1[base::intersect(which(Label_Cluster_real == 6),
                                                  which(LabelSourceRegion == "MOs"))]

laminar_MOs_7 = laminar_neuron$V1[base::intersect(which(Label_Cluster_real == 7),
                                                  which(LabelSourceRegion == "MOs"))]





## get Label_Source and Label_Cluster
Height = 2.8
K = c(3, 3, 3, 3, 1, 1, 3, 5, 5, 6, 3)

Label_Source = matrix(data = 0, nrow = NumNeuron, ncol = 1)
rownames(Label_Source) = as.character(proj_name_neuron[,1])

Label_Cluster = matrix(data = 0, nrow = NumNeuron, ncol = 1)
rownames(Label_Cluster) = as.character(proj_name_neuron[,1])

for(iSource in 1:NumSourceRegion) {
  tmpSource = SourceRegion[iSource]
  tmpDend = rowDend[[tmpSource]]
  tmpClusterLabel =cutree(tmpDend, k = K[iSource])
  tmpNumCluster = length(unique(tmpClusterLabel))
  
  for(iCluster in 1:tmpNumCluster) {
    tmpName = names(tmpClusterLabel[which(tmpClusterLabel == iCluster)])
    
    Label_Source[tmpName, ] = tmpSource
    Label_Cluster[tmpName, ] = iCluster
  }
}


# Calculate mean projection strength
proj_subtype_mean_value = matrix(data = 0, nrow = 100, ncol = NumRegion)
proj_subtype_mean_name = matrix(data = 0, nrow = 100, ncol = 1)

counter_subtype = 0

for(iSource in 1:NumSourceRegion) {
  tmpSource = SourceRegion[iSource]
  tmpClusterLabel = Label_Cluster[which(Label_Source == tmpSource),]
  tmpNumCluster = length(unique(tmpClusterLabel))
  
  for(iCluster in 1:tmpNumCluster) {
    tmpName = names(tmpClusterLabel[which(tmpClusterLabel == iCluster)])
    tmpProjMatrix = proj_as_matrix[tmpName,]
    
    # mean projection strength
    tmpProjMean = colMeans(tmpProjMatrix)
    
    counter_subtype = counter_subtype + 1
    proj_subtype_mean_value[counter_subtype,] = tmpProjMean
    proj_subtype_mean_name[counter_subtype,1] = paste0(tmpSource, "_", iCluster)
  }
}

proj_subtype_mean_value = proj_subtype_mean_value[1:counter_subtype,]
proj_subtype_mean_name = proj_subtype_mean_name[1:counter_subtype,]

rownames(proj_subtype_mean_value) = proj_subtype_mean_name
colnames(proj_subtype_mean_value) = as.character(proj_name_region[,1])












## get mean projection matrix for each neuronal subtype
# Height = 2.5
# 
# proj_subtype_mean_value = matrix(data = 0, nrow = 100, ncol = NumRegion)
# proj_subtype_mean_name = matrix(data = 0, nrow = 100, ncol = 1)
# 
# counter_subtype = 0
# 
# mesh_PFC= readOBJ("../IntraPFC_connection/result/label_PFC_volume_filled_25um.nrrd.obj")
# 
# open3d()
# mfrow3d(3, 4)
# 
# for(iSource in 1:NumSourceRegion) {
#   #  next3d()
#   
#   tmpSource = SourceRegion[iSource]
#   tmpDend = rowDend[[tmpSource]]
#   tmpOrder = rowOrder[[tmpSource]]
#   
#   tmpClusterLabel = cutree(tmpDend, h = Height)
#   
#   tmpNumCluster = length(unique(tmpClusterLabel))
#   
#   for(iCluster in 1:tmpNumCluster) {
#     tmpName = names(tmpClusterLabel[which(tmpClusterLabel == iCluster)])
#     tmpProjMatrix = proj_as_matrix[tmpName,]
#     
#     # visualization
#     # tmpSWCPath = SWCPath[tmpName,]
#     # tmpNeuronList = read.neurons(unname(tmpSWCPath), neuronnames = tmpName)
#     # 
#     # tmpNeuronList_flipped = func_flipRight2Left(tmpNeuronList, tmpName)
#     # 
#     # plot3d(tmpNeuronList_flipped, col = color[iCluster])
#     # shade3d(mesh_PFC, col = "grey", alpha = 0.2)
#     # 
#     # mean projection strength
#     tmpProjMean = colMeans(tmpProjMatrix)
#     
#     counter_subtype = counter_subtype + 1
#     proj_subtype_mean_value[counter_subtype,] = tmpProjMean
#     proj_subtype_mean_name[counter_subtype,1] = paste0(tmpSource, "_", iCluster)
#   }
# }
# 
# proj_subtype_mean_value = proj_subtype_mean_value[1:counter_subtype,]
# proj_subtype_mean_name = proj_subtype_mean_name[1:counter_subtype,]
# 
# rownames(proj_subtype_mean_value) = proj_subtype_mean_name
# colnames(proj_subtype_mean_value) = as.character(proj_name_region[,1])


ht_subtype = Heatmap(matrix = proj_subtype_mean_value[, c(seq(1,22,1),24)],
                     name = "Normalized strength",
                     heatmap_legend_param	= list(title_gp = gpar(fontsize = 7),
                                                 labels_gp = gpar(fontsize = 7)),
                     
                     #row_split = str_split_fixed(rownames(proj_subtype_mean_value), "_", n = 2)[,1],
                     
                     row_title = NULL,
                     
                     cluster_rows = TRUE,
                     clustering_distance_rows = "spearman",
                     clustering_method_rows = "ward.D2",
                     
                     cluster_columns = TRUE,
                     clustering_distance_columns = "spearman",
                     clustering_method_columns = "ward.D2",
                     
                     show_row_names = TRUE,
                     show_column_names = TRUE,
                     
                     width = unit(9, "cm"), 
                     height = unit(9, "cm"),
                     
                     row_names_side = "left",
                     row_names_rot = 0,
                     row_names_gp = gpar(fontsize = 5),
                     
                     column_names_side = "top",
                     column_names_rot = 90,
                     column_names_gp = gpar(fontsize = 5),
                     use_raster = TRUE
)

draw(ht_subtype)



## compare soma distribution
plotlist = vector('list', NumSourceRegion*3)

for(iSource in 1:NumSourceRegion) {
  tmpSource = SourceRegion[iSource]
  
  tmpValue = soma_position[which(Label_Source == tmpSource),]
  
  tmpGroup = Label_Cluster[which(Label_Source == tmpSource)]
  
  tmpValue$Group = as.factor(paste0(tmpSource, '_', tmpGroup))
  
  bp_X = ggboxplot(data = tmpValue, x = "Group", y = "V1",
                   font.label = list(size = 7, color = "black", face = "plain", family = NULL)) +
    geom_hline(yintercept = mean(tmpValue$V1), linetype = 2) +
    stat_compare_means(label = "p.signif", method = "wilcox.test", ref.group = ".all.", size = 5) +
    theme(axis.text = element_text(size = 5), 
          axis.title.x = element_blank(),
          axis.title.y = element_blank(),
          line = element_line(size = 0.25))
  
  bp_Y = ggboxplot(data = tmpValue, x = "Group", y = "V2",
                   font.label = list(size = 7, color = "black", face = "plain", family = NULL)) +
    geom_hline(yintercept = mean(tmpValue$V2), linetype = 2) +
    stat_compare_means(label = "p.signif", method = "wilcox.test", ref.group = ".all.") +
    theme(axis.text = element_text(size = 5), 
          axis.title.x = element_blank(),
          axis.title.y = element_blank(),
          line = element_line(size = 0.25))
  
  bp_Z = ggboxplot(data = tmpValue, x = "Group", y = "V3",
                   font.label = list(size = 7, color = "black", face = "plain", family = NULL)) +
    geom_hline(yintercept = mean(tmpValue$V3), linetype = 2) +
    stat_compare_means(label = "p.signif", method = "wilcox.test", ref.group = ".all.") +
    theme(axis.text = element_text(size = 5), 
          axis.title.x = element_blank(),
          axis.title.y = element_blank(),
          line = element_line(size = 0.25))
  
  plotlist[[3*iSource-2]] = bp_X
  plotlist[[3*iSource-1]] = bp_Y
  plotlist[[3*iSource-0]] = bp_Z
}

ggarrange(plotlist = plotlist, nrow = NumSourceRegion, ncol = 3)






## Write label of cortical columns 
NumColumn = 40
out = array(data = 0, dim = c(NumColumn, NumCluster))

for (iCluster in 1:NumCluster) {
  out[, iCluster] = tabulate(label_CC_neuron[which(Label_Cluster_real == iCluster)], NumColumn)
}

write.table(x = out, 
            file = "./level_3_label_CC_clusters.csv", 
            sep = ",", 
            row.names = FALSE, 
            col.names = FALSE)


# ACA
index_ACA_5 = base::intersect(which(Label_Cluster_real == 5), 
                               base::union(which(LabelSourceRegion == "ACAd"), which(LabelSourceRegion == "ACAv")))
index_ACA_6 = base::intersect(which(Label_Cluster_real == 6), 
                               base::union(which(LabelSourceRegion == "ACAd"), which(LabelSourceRegion == "ACAv")))

soma_ACA = rbind(as.matrix(laminar_neuron[index_ACA_5]),
                  as.matrix(laminar_neuron[index_ACA_6]))

label_ACA = c(rep("IT-5", length(index_ACA_5)), 
               rep("IT-6", length(index_ACA_6)))

laminar_ACA = data.frame(laminar = soma_ACA,
                          group = label_ACA)

ggboxplot(laminar_ACA, x = "group", y = "laminar", orientation = "vertical") +
  #scale_x_discrete(limits = rev(levels(as.factor(symmetry_group)))) +
  stat_compare_means(
    #method = "wilcox.test"
    )


open3d()
points3d(soma_position[index_ACA_5,], col = "red", size = 20)
points3d(soma_position[index_ACA_6,], col = "green", size = 20)

shade3d(meshlist[[2]], col = "grey", alpha = 0.2)
shade3d(meshlist[[3]], col = "grey", alpha = 0.2)

#shade3d(mesh_root, col = "grey", alpha = 0.1)
#decorate3d()
view3d(userMatrix = userMatrix_LM)






# AI IT-1, IT-2 and IT-4 neurons
index_AI_1 = base::intersect(which(Label_Cluster_real == 1), 
                              base::union(which(LabelSourceRegion == "AIv"), 
                                          which(LabelSourceRegion == "AId")))
index_AI_2 = base::intersect(which(Label_Cluster_real == 2), 
                              base::union(which(LabelSourceRegion == "AIv"), 
                                          which(LabelSourceRegion == "AId")))
index_AI_4 = base::intersect(which(Label_Cluster_real == 4), 
                              base::union(which(LabelSourceRegion == "AIv"),
                                          which(LabelSourceRegion == "AId")))

laminar_AI = rbind(as.matrix(laminar_neuron[index_AI_1]),
                  as.matrix(laminar_neuron[index_AI_2]),
                 as.matrix(laminar_neuron[index_AI_4]))

label_AI = c(rep("IT-1", length(index_AI_1)), 
               rep("IT-2", length(index_AI_2)),
              rep("IT-4", length(index_AI_4)))

df_laminar_AI = data.frame(value = laminar_AI, group = label_AI)



##
ggboxplot(df_laminar_AI, x = "group", y = "value", orientation = "vertical") +
  #scale_x_discrete(limits = rev(levels(as.factor(symmetry_group)))) +
  stat_compare_means(
    #method = "wilcox.test"
  )


## laminar position
ggplot(df_laminar_AI, aes(x = value, y = group)) +
  geom_density_ridges(scale = 0.9,
                      size = 0.1, 
                      jittered_points = FALSE,
                      position = position_points_jitter(width = 0.05, height = 0),
                      point_shape = '|', point_size = 3, point_alpha = 1, alpha = 0.3)  +
  #scale_x_continuous(expand = c(0.01,0.7)) +
  #scale_y_discrete(expand = c(0.01,0.7), limits = rev(levels(as.factor(group)))) +
  #scale_fill_manual(values = color, labels = seq(1, 14, by = 1)) +
  theme_ridges(grid = FALSE)+
  theme(axis.text = element_text(size = 7), line = element_line(size = 0.25))+
  xlim(c(0,1)) +
  xlab(NULL) +
  ylab(NULL)
#scale_fill_cyclical(values = clusterColorList)





AId_ipsi = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/104_AId.obj")
AId5_ipsi = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/1101_AId5.obj")

AIv_ipsi = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/119_AIv.obj")
AIv5_ipsi = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/800_AIv5.obj")



open3d()
points3d(soma_position[index_AI_1,], col = "red", size = 20)
points3d(soma_position[index_AI_2,], col = "green", size = 20)
points3d(soma_position[index_AI_4,], col = "blue", size = 20)


shade3d(AId_ipsi, col = "grey", alpha = 0.2)
shade3d(AIv_ipsi, col = "grey", alpha = 0.2)

# wire3d(AId5_ipsi, col = "grey", alpha = 0.2)
# wire3d(AIv5_ipsi, col = "grey", alpha = 0.2)



#shade3d(mesh_root, col = "grey", alpha = 0.1)
rgl.viewpoint(fov = 0)
view3d(userMatrix = userMatrix_LM)


soma_position_AI_DV = rbind(as.matrix(soma_position[index_AI_1,2]),
                         as.matrix(soma_position[index_AI_2,2]),
                         as.matrix(soma_position[index_AI_4,2]))

DV_AI = data.frame(value = soma_position_AI_DV, group = label_AI)

ggboxplot(DV_AI, x = "group", y = "value", orientation = "vertical") +
  #scale_x_discrete(limits = rev(levels(as.factor(symmetry_group)))) +
  stat_compare_means(
    #method = "wilcox.test"
  )

## statistics for dorsal-ventral axis
xx=ggpubr::compare_means(formula = value ~ group, 
                         data = DV_AI, 
                         methods = "kruskal.test",
                         p.adjust.method = "none")


wilcox.test(x = soma_position[index_AI_1,2], 
            y = soma_position[index_AI_2,2],
            alternative = "two.sided")





##
ggplot(test, aes(x = position, y = group, color = type, fill = type)) +
  geom_density_ridges(scale = 0.9,
                      size = 0.1, 
                      jittered_points = FALSE,
                      position = position_points_jitter(width = 0.05, height = 0),
                      point_shape = '|', point_size = 3, point_alpha = 1, alpha = 0.3)  +
  #scale_x_continuous(expand = c(0.01,0.7)) +
  scale_y_discrete(expand = c(0.01,0.7), limits = rev(levels(as.factor(group)))) +
  #scale_fill_manual(values = color, labels = seq(1, 14, by = 1)) +
  theme_ridges(grid = FALSE)+
  theme(axis.text = element_text(size = 7), line = element_line(size = 0.25))+
  xlim(c(0,1)) +
  xlab(NULL) +
  ylab(NULL)
  #scale_fill_cyclical(values = clusterColorList)


ggsave2("/home/admin/shared/figure5/laminar_position.pdf", 
        scale = 1, width = 30, height = 40, units = "mm", dpi = 300)


## statistical test
sta_position = c()
sta_group = c()

counterGroup = 0;
for (iCluster in 1:NumClusters) {
  tmpPosition = laminar_neuron$V1[which(iCluster == clusters)]
  
  counterGroup = counterGroup + 1
  sta_group = c(sta_group, rep(counterGroup, length(tmpPosition)))
  sta_position = c(sta_position, tmpPosition)
}

sta_group = c(sta_group, rep(0, NumNeuron))
sta_position = c(sta_position, laminar_neuron$V1)

sta_laminarPosition = data.frame(sta_group = as.factor(sta_group),
                                 sta_position = sta_position)


xx=ggpubr::compare_means(formula = sta_position~sta_group, 
                      data = sta_laminarPosition, 
                      methods = "kruskal.test",
                      ref.group = "0", 
                      p.adjust.method = "none")


## Calculation of symmetry for each cluster
symmetry_value = array(0, dim = c(NumNeuron, 1))
symmetry_group = array(0, dim = c(NumNeuron, 1))

for (iCluster in 1:NumClusters) {
  tmpNeuronIndexList = which(iCluster == clusters)
  
  for (iNeuron in 1:length(tmpNeuronIndexList)) {
    tmpNeuronIndex = tmpNeuronIndexList[iNeuron]
    
    tmpLeft = as.numeric(proj_as_matrix[tmpNeuronIndex,1:(NumRegion/2)] > 0)
    tmpRight = as.numeric(proj_as_matrix[tmpNeuronIndex, (NumRegion/2+1):NumRegion] > 0)
    
    tmpIntersection = as.numeric(tmpLeft & tmpRight)
    tmpUnion = as.numeric(tmpLeft | tmpRight)
    
    tmpJaccard = length(which(tmpIntersection == 1)) / length(which(tmpUnion == 1))
    #tmpCor = cor(tmpLeft, tmpRight)
    
    symmetry_value[tmpNeuronIndex,1] = tmpJaccard
    symmetry_group[tmpNeuronIndex,1] = iCluster
  }
}

symmetry = data.frame(cluster = as.factor(symmetry_group), value = symmetry_value)

ggboxplot(symmetry, x = "cluster", y = "value", orientation = "horizontal") +
  scale_x_discrete(limits = rev(levels(as.factor(symmetry_group)))) +
  stat_compare_means(label = "p.signif", method = "t.test", ref.group = ".all.")



  
ggplot(symmetry, aes(cluster, value)) +
  geom_boxplot()


## Choose number of clusters
dist = Dist(x = proj_as_matrix, method = "euclidean")

## Determine the number of clusters
nc = NbClust(data = proj_as_matrix, 
             diss = dist, 
             distance = NULL,
             min.nc = 4,
             max.nc = 12, 
             method = "ward.D2",
             index = "all"
)

nc$All.index[,"Pseudot2"]
nc$Best.partition


## Write clusters and name of neurons (Sang LIU)
out_cluster = as.matrix(clusters)
rownames(out_cluster) = proj_name_neuron[,1]

write.table(x = out_cluster, 
            file = "/tmp/classification_IT.csv", 
            sep = ",", 
            row.names = TRUE, 
            col.names = FALSE)

## diversity

matrix_diversity = array(data = 0, dim = c(NumSourceRegion, 20))

##
for(iNumCluster in 2:20) {


NumCluster = iNumCluster

Label_Cluster = cutree(tree = columnDend, k = NumCluster)

labelOnDend = unique(Label_Cluster[columnOrder])


tmpLabel_Cluster = Label_Cluster

for(iLabel in 1:NumCluster) {
  tmpLabel_Cluster[which(Label_Cluster == labelOnDend[iLabel])] = iLabel
}

## regional distribution
contingency_table = func_getClusterContingencyTable(NumCluster, tmpLabel_Cluster, proj_name_neuron, NeuronDistribution)
distribution_table = apply(contingency_table, 2, function(x) x/sum(x))

test_diversity = array(data = 0, dim = c(NumSourceRegion, 1))
for(iSourceRegion in 1:11) {
  tmp = 0
  for(iClusterLabel in 1:NumCluster) {
    tmpRatio = distribution_table[iClusterLabel, iSourceRegion]
    if(tmpRatio == 0)
      next
    tmp = tmp + (-tmpRatio * log(tmpRatio))
  }
  test_diversity[iSourceRegion,1] = tmp
}

matrix_diversity[,iNumCluster] = test_diversity[,1]/log(iNumCluster)

}
rownames(matrix_diversity) = SourceRegion

## Plot curve
#dev.off()
cluster = 1:20
plot(cluster, matrix_diversity[1,], col = ColorSourceRegion[1])
for(iSourceRegion in 2:11) {
  lines(cluster, matrix_diversity[iSourceRegion,], col = ColorSourceRegion[iSourceRegion])
}
#legend("topleft", legend = SourceRegion, col = ColorSourceRegion, lty = 1, lwd = 10)

boxplot(t(matrix_diversity))


## figure


open3d()

shade3d(meshlist[[1]], color = "green", alpha = 1)
shade3d(mesh_root, color = "grey", alpha = 1)


rgl.viewpoint(fov = 0)



## 

mesh_RSP = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/254_RSP_v2.obj")
mesh_VIS = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/669_VIS.obj")
mesh_PTLp = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/22_PTLp.obj")

mesh_MOp = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/985_MOp.obj")
mesh_SSs = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/378_SSs.obj")
mesh_SSp = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/322_SSp.obj")

mesh_GU = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/1057_GU.obj")
mesh_AIp = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/111_AIp.obj")
mesh_VISC = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/677_VISC.obj")

mesh_ENT = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/909_ENT.obj")
mesh_ECT = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/895_ECT.obj")
mesh_PERI = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/922_PERI.obj")

mesh_TEa = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/541_TEa.obj")
mesh_AUD = readOBJ("/home/admin/Documents/YanLab/database_neuron/ABA_mesh_left_hemisphere/247_AUD.obj")



open3d()
shade3d(mesh_root, color = "grey", alpha = 0.2)

shade3d(mesh_RSP, color = "#29abe2", alpha = 1)
shade3d(mesh_VIS, color = "#29abe2", alpha = 1)
shade3d(mesh_PTLp, color = "#29abe2", alpha = 1)

shade3d(mesh_MOp, color = "#00ff00", alpha = 1)
shade3d(mesh_SSp, color = "#00ff00", alpha = 1)
shade3d(mesh_SSs, color = "#00ff00", alpha = 1)

shade3d(mesh_GU, color = "#f7931e", alpha = 1)
shade3d(mesh_AIp, color = "#f7931e", alpha = 1)
shade3d(mesh_VISC, color = "#f7931e", alpha = 1)

shade3d(mesh_ENT, color = "#ff00ff", alpha = 1)
shade3d(mesh_ECT, color = "#ff00ff", alpha = 1)
shade3d(mesh_PERI, color = "#ff00ff", alpha = 1)

shade3d(mesh_TEa, color = "#ffff00", alpha = 1)
shade3d(mesh_AUD, color = "#ffff00", alpha = 1)

rgl.viewpoint(fov = 0)
